/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.component.framework;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.List;

import javax.jbi.JBIException;
import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.ow2.petals.component.framework.jbidescriptor.CDKJBIDescriptorException;
import org.ow2.petals.component.framework.jbidescriptor.JBIDescriptorBuilder;
import org.ow2.petals.component.framework.mbean.MBeanHelper;

import static org.ow2.petals.component.framework.Constants.Component.JBI_XML;
import static org.ow2.petals.component.framework.Constants.Component.META_INF;

/**
 * A default bootstrap. This bootstrap class register an MBean object into the
 * JMX server that will be available only during the bootstrap phase. The
 * developer only needs to override methods @ code getMethodList()} to set the
 * list of method that he want to expose via JMX.
 * 
 * @author chamerling - EBM WebSourcing
 * @author rnaudin - EBM WebSourcing
 * 
 */
public class DefaultBootstrap extends Bootstrap {

    /**
     * The bootstrap MBean name
     */
    protected ObjectName mbeanName;

    /**
     * Returns null by default as this class does not provides JMX extension
     * MBean
     * 
     * @see javax.jbi.component.ComponentLifeCycle#getExtensionMBeanName()
     */
    @Override
    public ObjectName getExtensionMBeanName() {
        return this.mbeanName;
    }

    /**
     * Persist the configuration done via the extension Mbean.
     * 
     * @see javax.jbi.component.Bootstrap#onInstall()
     */
    @Override
    public void onInstall() throws JBIException {
        final File jbiXmlFile = new File(getInstallContext().getInstallRoot() + File.separator
                + META_INF + File.separator + JBI_XML);

        try {
            JBIDescriptorBuilder.buildXmlJBIdescriptor(getJbiComponentConfiguration(),
                    new FileOutputStream(jbiXmlFile));
        } catch (FileNotFoundException e) {
            throw new JBIException("Failed to persist the installation configuration", e);
        } catch (CDKJBIDescriptorException e) {
            throw new JBIException("Failed to persist the installation configuration", e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.Bootstrap#cleanUp()
     */
    @Override
    public void cleanUp() throws JBIException {
        try {
            if (this.mbeanName != null) {
                MBeanServer server = installContext.getContext().getMBeanServer();
                if (server == null) {
                    throw new JBIException("MBean server is null !");
                }
                if (server.isRegistered(mbeanName)) {
                    server.unregisterMBean(mbeanName);
                }
            }
        } catch (Exception e) {
            throw new JBIException("Error bootstrap clean up", e);
        }
    }

    /**
     * Registers the Extension MBean into the JMX server.
     * 
     * @see org.ow2.petals.component.framework.Bootstrap#doInit()
     */
    protected void doInit() throws Exception {

        this.mbeanName = this.installContext.getContext().getMBeanNames()
                .createCustomComponentMBeanName("bootstrap_" + installContext.getComponentName());

        MBeanHelper.registerMBean(this.installContext.getContext().getMBeanServer(),
                this.mbeanName, new ExtensionMBean(this));
    }

    /**
     * Method to be overridden if the component implementation needs to expose
     * methods via JMX. This method is called by the dynamic MBean.
     * 
     * @param componentProperties
     * @param methods
     */
    protected List<String> getMethodList() {
        return new ArrayList<String>();
    }

    /**
     * Method to be overridden if the component implementation needs to expose
     * attributes via JMX. This method is called by the dynamic MBean.
     * 
     * @param componentProperties
     * @param methods
     */
    protected List<String> getAttributeList() {
        return new ArrayList<String>();
    }
}
